@@ -7,6 +7,7 @@
 #include "numpy/npy_common.h"
 #include "numpy/halffloat.h"
 #include "numpy/ndarrayobject.h"
+#include "numpy/arrayscalars.h"
 
 int lsize = NPY_SIZEOF_LONG;
 
@@ -79,6 +80,52 @@ error:
     return 0;
 };
 
+long getNumpyL(PyObject *obj) {
+    short s;
+    int i;
+    long l;
+    long long ll;
+    unsigned short us;
+    unsigned int ui;
+    unsigned long ul;
+    unsigned long long ull;
+    
+    if(!PyArray_IsIntegerScalar(obj)) {
+        PyErr_SetString(PyExc_RuntimeError, "Received non-Integer scalar type for conversion to long!\n");
+        return 0;
+    }
+
+    if(PyArray_IsScalar(obj, Short)) {
+        s = ((PyShortScalarObject *)obj)->obval;
+        l = s;
+    } else if(PyArray_IsScalar(obj, Int)) {
+        i = ((PyLongScalarObject *)obj)->obval;
+        l = i;
+    } else if(PyArray_IsScalar(obj, Long)) {
+        l = ((PyLongScalarObject *)obj)->obval;
+    } else if(PyArray_IsScalar(obj, LongLong)) {
+        ll = ((PyLongScalarObject *)obj)->obval;
+        l = ll;
+    } else if(PyArray_IsScalar(obj, UShort)) {
+        us = ((PyLongScalarObject *)obj)->obval;
+        l = us;
+    } else if(PyArray_IsScalar(obj, UInt)) {
+        ui = ((PyLongScalarObject *)obj)->obval;
+        l = ui;
+    } else if(PyArray_IsScalar(obj, ULong)) {
+        ul = ((PyLongScalarObject *)obj)->obval;
+        l = ul;
+    } else if(PyArray_IsScalar(obj, ULongLong)) {
+        ull = ((PyLongScalarObject *)obj)->obval;
+        l = ull;
+    } else {
+        PyErr_SetString(PyExc_RuntimeError, "Received unknown scalar type for conversion to long!\n");
+        return 0;
+    }
+
+    return l;
+}
+
 //Raises an exception on error, which should be checked
 float getNumpyF(PyArrayObject *obj, Py_ssize_t i) {
     int dtype;
@@ -119,6 +166,7 @@ error:
     return 0;
 }
 
+//The calling function needs to free the result
 char *getNumpyStr(PyArrayObject *obj, Py_ssize_t i) {
     char *p , *o = NULL;
     npy_intp stride, j;
@@ -137,7 +185,7 @@ char *getNumpyStr(PyArrayObject *obj, Py_ssize_t i) {
         return o;
     case NPY_UNICODE:
         o = calloc(1, stride/4 + 1);
-        for(j=0; j<stride/4; j++) o[j] = (char) ((uint32_t*)p)[4*j];
+        for(j=0; j<stride/4; j++) o[j] = (char) ((uint32_t*)p)[j];
         return o;
     default:
         PyErr_SetString(PyExc_RuntimeError, "Received unknown data type!\n");
@@ -147,6 +195,25 @@ char *getNumpyStr(PyArrayObject *obj, Py_ssize_t i) {
 }
 #endif
 
+//Return 1 if there are any entries at all
+int hasEntries(bigWigFile_t *bw) {
+    if(bw->hdr->nBasesCovered > 0) return 1;
+    return 0;
+}
+
+PyObject* pyBwEnter(pyBigWigFile_t*self, PyObject *args) {
+    bigWigFile_t *bw = self->bw;
+
+    if(!bw) {
+        PyErr_SetString(PyExc_RuntimeError, "The bigWig file handle is not opened!");
+        return NULL;
+    }
+
+    Py_INCREF(self);
+
+    return (PyObject*) self;
+}
+
 PyObject* pyBwOpen(PyObject *self, PyObject *pyFname) {
     char *fname = NULL;
     char *mode = "r";
@@ -161,13 +228,19 @@ PyObject* pyBwOpen(PyObject *self, PyObject *pyFname) {
     } else {
         bw = bbOpen(fname, NULL);
     }
-    if(!bw) goto error;
+    if(!bw) {
+        fprintf(stderr, "[pyBwOpen] bw is NULL!\n");
+        goto error;
+    }
     if(!mode || !strchr(mode, 'w')) {
         if(!bw->cl) goto error;
     }
 
     pybw = PyObject_New(pyBigWigFile_t, &bigWigFile);
-    if(!pybw) goto error;
+    if(!pybw) {
+        fprintf(stderr, "[pyBwOpen] PyObject_New() returned NULL (out of memory?)!\n");
+        goto error;
+    }
     pybw->bw = bw;
     pybw->lastTid = -1;
     pybw->lastType = -1;
@@ -199,6 +272,11 @@ static PyObject *pyBwGetHeader(pyBigWigFile_t *self, PyObject *args) {
     bigWigFile_t *bw = self->bw;
     PyObject *ret, *val;
 
+    if(!bw) {
+        PyErr_SetString(PyExc_RuntimeError, "The bigWig file handle is not opened!");
+        return NULL;
+    }
+
     ret = PyDict_New();
     val = PyLong_FromUnsignedLong(bw->hdr->version);
     if(PyDict_SetItemString(ret, "version", val) == -1) goto error;
@@ -238,6 +316,11 @@ static PyObject *pyBwGetChroms(pyBigWigFile_t *self, PyObject *args) {
     char *chrom = NULL;
     uint32_t i;
 
+    if(!bw) {
+        PyErr_SetString(PyExc_RuntimeError, "The bigWig file handle is not opened!");
+        return NULL;
+    }
+
     if(!(PyArg_ParseTuple(args, "|s", &chrom)) || !chrom) {
         ret = PyDict_New();
         for(i=0; i<bw->cl->nKeys; i++) {
@@ -276,6 +359,7 @@ enum bwStatsType char2enum(char *s) {
     if(strcmp(s, "min") == 0) return min;
     if(strcmp(s, "cov") == 0) return cov;
     if(strcmp(s, "coverage") == 0) return cov;
+    if(strcmp(s, "sum") == 0) return sum;
     return -1;
 };
 
@@ -287,16 +371,21 @@ static PyObject *pyBwGetStats(pyBigWigFile_t *self, PyObject *args, PyObject *kw
     unsigned long startl = 0, endl = -1;
     static char *kwd_list[] = {"chrom", "start", "end", "type", "nBins", "exact", NULL};
     char *chrom, *type = "mean";
-    PyObject *ret, *exact = Py_False;
+    PyObject *ret, *exact = Py_False, *starto = NULL, *endo = NULL;
     int i, nBins = 1;
     errno = 0; //In the off-chance that something elsewhere got an error and didn't clear it...
 
+    if(!bw) {
+        PyErr_SetString(PyExc_RuntimeError, "The bigWig file handle is not open!");
+        return NULL;
+    }
+
     if(bw->type == 1) {
         PyErr_SetString(PyExc_RuntimeError, "bigBed files have no statistics!");
         return NULL;
     }
 
-    if(!PyArg_ParseTupleAndKeywords(args, kwds, "s|kksiO", kwd_list, &chrom, &startl, &endl, &type, &nBins, &exact)) {
+    if(!PyArg_ParseTupleAndKeywords(args, kwds, "s|OOsiO", kwd_list, &chrom, &starto, &endo, &type, &nBins, &exact)) {
         PyErr_SetString(PyExc_RuntimeError, "You must supply at least a chromosome!");
         return NULL;
     }
@@ -305,6 +394,43 @@ static PyObject *pyBwGetStats(pyBigWigFile_t *self, PyObject *args, PyObject *kw
     if(!nBins) nBins = 1; //For some reason, not specifying this overrides the default!
     if(!type) type = "mean";
     tid = bwGetTid(bw, chrom);
+
+    if(starto) {
+#ifdef WITHNUMPY
+        if(PyArray_IsScalar(starto, Integer)) {
+            startl = (long) getNumpyL(starto);
+        } else 
+#endif
+        if(PyLong_Check(starto)) {
+            startl = PyLong_AsLong(starto);
+#if PY_MAJOR_VERSION < 3
+        } else if(PyInt_Check(starto)) {
+            startl = PyInt_AsLong(starto);
+#endif
+        } else {
+            PyErr_SetString(PyExc_RuntimeError, "The start coordinate must be a number!");
+            return NULL;
+        }
+    }
+
+    if(endo) {
+#ifdef WITHNUMPY
+        if(PyArray_IsScalar(endo, Integer)) {
+            endl = (long) getNumpyL(endo);
+        } else 
+#endif
+        if(PyLong_Check(endo)) {
+            endl = PyLong_AsLong(endo);
+#if PY_MAJOR_VERSION < 3
+        } else if(PyInt_Check(endo)) {
+            endl = PyInt_AsLong(endo);
+#endif
+        } else {
+            PyErr_SetString(PyExc_RuntimeError, "The end coordinate must be a number!");
+            return NULL;
+        }
+    }
+
     if(endl == (unsigned long) -1 && tid != (uint32_t) -1) endl = bw->cl->len[tid];
     if(tid == (uint32_t) -1 || startl > end || endl > end) {
         PyErr_SetString(PyExc_RuntimeError, "Invalid interval bounds!");
@@ -322,6 +448,16 @@ static PyObject *pyBwGetStats(pyBigWigFile_t *self, PyObject *args, PyObject *kw
         return NULL;
     }
 
+    //Return a list of None if there are no entries at all
+    if(!hasEntries(bw)) {
+        ret = PyList_New(nBins);
+        for(i=0; i<nBins; i++) {
+            Py_INCREF(Py_None);
+            PyList_SetItem(ret, i, Py_None);
+        }
+        return ret;
+    }
+
     //Get the actual statistics
     if(exact == Py_True) {
         val = bwStatsFromFull(bw, chrom, start, end, nBins, char2enum(type));
@@ -360,9 +496,14 @@ static PyObject *pyBwGetValues(pyBigWigFile_t *self, PyObject *args) {
     uint32_t start, end = -1, tid;
     unsigned long startl, endl;
     char *chrom;
-    PyObject *ret;
+    PyObject *ret, *starto = NULL, *endo = NULL;
     bwOverlappingIntervals_t *o;
 
+    if(!bw) {
+        PyErr_SetString(PyExc_RuntimeError, "The bigWig file handle is not open!");
+        return NULL;
+    }
+
     if(bw->type == 1) {
         PyErr_SetString(PyExc_RuntimeError, "bigBed files have no values! Use 'entries' instead.");
         return NULL;
@@ -372,20 +513,54 @@ static PyObject *pyBwGetValues(pyBigWigFile_t *self, PyObject *args) {
     static char *kwd_list[] = {"chrom", "start", "end", "numpy", NULL};
     PyObject *outputNumpy = Py_False;
 
-    if(!PyArg_ParseTupleAndKeywords(args, kwds, "skk|O", kwd_list, &chrom, &startl, &endl, &outputNumpy)) {
+    if(!PyArg_ParseTupleAndKeywords(args, kwds, "sOO|O", kwd_list, &chrom, &starto, &endo, &outputNumpy)) {
 #else
-    if(!PyArg_ParseTuple(args, "skk", &chrom, &startl, &endl)) {
+    if(!PyArg_ParseTuple(args, "sOO", &chrom, &starto, &endo)) {
 #endif
         PyErr_SetString(PyExc_RuntimeError, "You must supply a chromosome, start and end position.\n");
         return NULL;
     }
 
     tid = bwGetTid(bw, chrom);
+
+#ifdef WITHNUMPY
+    if(PyArray_IsScalar(starto, Integer)) {
+        startl = (long) getNumpyL(starto);
+    } else
+#endif
+    if(PyLong_Check(starto)) {
+        startl = PyLong_AsLong(starto);
+#if PY_MAJOR_VERSION < 3
+    } else if(PyInt_Check(starto)) {
+        startl = PyInt_AsLong(starto);
+#endif
+    } else {
+        PyErr_SetString(PyExc_RuntimeError, "The start coordinate must be a number!");
+        return NULL;
+    }
+
+#ifdef WITHNUMPY
+    if(PyArray_IsScalar(endo, Integer)) {
+        endl = (long) getNumpyL(endo);
+    } else
+#endif
+    if(PyLong_Check(endo)) {
+        endl = PyLong_AsLong(endo);
+#if PY_MAJOR_VERSION < 3
+    } else if(PyInt_Check(endo)) {
+        endl = PyInt_AsLong(endo);
+#endif
+    } else {
+        PyErr_SetString(PyExc_RuntimeError, "The end coordinate must be a number!");
+        return NULL;
+    }
+
     if(endl == (unsigned long) -1 && tid != (uint32_t) -1) endl = bw->cl->len[tid];
     if(tid == (uint32_t) -1 || startl > end || endl > end) {
         PyErr_SetString(PyExc_RuntimeError, "Invalid interval bounds!");
         return NULL;
     }
+
     start = (uint32_t) startl;
     end = (uint32_t) endl;
     if(end <= start || end > bw->cl->len[tid] || start >= end) {
@@ -393,6 +568,18 @@ static PyObject *pyBwGetValues(pyBigWigFile_t *self, PyObject *args) {
         return NULL;
     }
 
+    if(!hasEntries(self->bw)) {
+#ifdef WITHNUMPY
+        if(outputNumpy == Py_True) {
+            return PyArray_SimpleNew(0, NULL, NPY_FLOAT);
+        } else {
+#endif
+            return PyList_New(0);
+#ifdef WITHNUMPY
+        }
+#endif
+    }
+
     o = bwGetValues(self->bw, chrom, start, end, 1);
     if(!o) {
         PyErr_SetString(PyExc_RuntimeError, "An error occurred while fetching values!");
@@ -427,14 +614,19 @@ static PyObject *pyBwGetIntervals(pyBigWigFile_t *self, PyObject *args, PyObject
     static char *kwd_list[] = {"chrom", "start", "end", NULL};
     bwOverlappingIntervals_t *intervals = NULL;
     char *chrom;
-    PyObject *ret;
+    PyObject *ret, *starto = NULL, *endo = NULL;
+
+    if(!bw) {
+        PyErr_SetString(PyExc_RuntimeError, "The bigWig file handle is not opened!");
+        return NULL;
+    }
 
     if(bw->type == 1) {
         PyErr_SetString(PyExc_RuntimeError, "bigBed files have no intervals! Use 'entries()' instead.");
         return NULL;
     }
 
-    if(!PyArg_ParseTupleAndKeywords(args, kwds, "s|kk", kwd_list, &chrom, &startl, &endl)) {
+    if(!PyArg_ParseTupleAndKeywords(args, kwds, "s|OO", kwd_list, &chrom, &starto, &endo)) {
         PyErr_SetString(PyExc_RuntimeError, "You must supply at least a chromosome.\n");
         return NULL;
     }
@@ -446,6 +638,43 @@ static PyObject *pyBwGetIntervals(pyBigWigFile_t *self, PyObject *args, PyObject
         PyErr_SetString(PyExc_RuntimeError, "Invalid interval bounds!");
         return NULL;
     }
+
+    if(starto) {
+#ifdef WITHNUMPY
+        if(PyArray_IsScalar(starto, Integer)) {
+            startl = (long) getNumpyL(starto);
+        } else
+#endif
+        if(PyLong_Check(starto)) {
+            startl = PyLong_AsLong(starto);
+#if PY_MAJOR_VERSION < 3
+        } else if(PyInt_Check(starto)) {
+            startl = PyInt_AsLong(starto);
+#endif
+        } else {
+            PyErr_SetString(PyExc_RuntimeError, "The start coordinate must be a number!");
+            return NULL;
+        }
+    }
+
+    if(endo) {
+#ifdef WITHNUMPY
+        if(PyArray_IsScalar(endo, Integer)) {
+            endl = (long) getNumpyL(endo);
+        } else
+#endif
+        if(PyLong_Check(endo)) {
+            endl = PyLong_AsLong(endo);
+#if PY_MAJOR_VERSION < 3
+        } else if(PyInt_Check(endo)) {
+            endl = PyInt_AsLong(endo);
+#endif
+        } else {
+            PyErr_SetString(PyExc_RuntimeError, "The end coordinate must be a number!");
+            return NULL;
+        }
+    }
+
     start = (uint32_t) startl;
     end = (uint32_t) endl;
     if(end <= start || end > bw->cl->len[tid] || start >= end) {
@@ -453,6 +682,12 @@ static PyObject *pyBwGetIntervals(pyBigWigFile_t *self, PyObject *args, PyObject
         return NULL;
     }
 
+    //Check for empty files
+    if(!hasEntries(bw)) {
+        Py_INCREF(Py_None);
+        return Py_None;
+    }
+
     //Get the intervals
     intervals = bwGetOverlappingIntervals(bw, chrom, start, end);
     if(!intervals) {
@@ -533,6 +768,11 @@ PyObject *pyBwAddHeader(pyBigWigFile_t *self, PyObject *args, PyObject *kwds) {
     PyObject *InputTuple = NULL, *tmpObject, *tmpObject2;
     Py_ssize_t i, pyLen;
 
+    if(!bw) {
+        PyErr_SetString(PyExc_RuntimeError, "The bigWig file handle is not open!");
+        return NULL;
+    }
+
     if(!PyArg_ParseTupleAndKeywords(args, kwds, "O|k", kwd_list, &InputTuple, &zoomTmp)) {
         PyErr_SetString(PyExc_RuntimeError, "Illegal arguments");
         return NULL;
@@ -824,7 +1064,7 @@ int canAppend(pyBigWigFile_t *self, int desiredType, PyObject *chroms, PyObject
     uint32_t tid, uspan, ustep, ustart;
     PyObject *tmp;
 #ifdef WITHNUMPY
-    void *foo;
+    char *chrom;
 #endif
 
     if(self->lastType == -1) return 0;
@@ -838,11 +1078,13 @@ int canAppend(pyBigWigFile_t *self, int desiredType, PyObject *chroms, PyObject
 #ifdef WITHNUMPY
         if(PyArray_Check(chroms)) sz = PyArray_Size(chroms);
 #endif
+
         for(i=0; i<sz; i++) {
 #ifdef WITHNUMPY
             if(PyArray_Check(chroms)) {
-                foo = PyArray_GETPTR1((PyArrayObject*)chroms, i);
-                tid = bwGetTid(bw, PyString_AsString(PyArray_GETITEM((PyArrayObject*)chroms, foo)));
+                chrom = getNumpyStr((PyArrayObject*)chroms, i);
+                tid = bwGetTid(bw, chrom);
+                free(chrom);
             } else {
 #endif
                 tmp = PyList_GetItem(chroms, i);
@@ -852,6 +1094,7 @@ int canAppend(pyBigWigFile_t *self, int desiredType, PyObject *chroms, PyObject
 #endif
             if(tid != (uint32_t) self->lastTid) return 0;
         }
+
 #ifdef WITHNUMPY
         if(PyArray_Check(starts)) {
             ustart = getNumpyU32((PyArrayObject*)starts, 0);
@@ -911,9 +1154,6 @@ int PyAddIntervals(pyBigWigFile_t *self, PyObject *chroms, PyObject *starts, PyO
     uint32_t n, *ustarts = NULL, *uends = NULL;
     float *fvalues = NULL;
     int rv;
-#ifdef WITHNUMPY
-    void *foo;
-#endif
 
     if(PyList_Check(starts)) sz = PyList_Size(starts);
 #ifdef WITHNUMPY
@@ -933,8 +1173,7 @@ int PyAddIntervals(pyBigWigFile_t *self, PyObject *chroms, PyObject *starts, PyO
             cchroms[i] = PyString_AsString(PyList_GetItem(chroms, i));
 #ifdef WITHNUMPY
         } else {
-            foo = PyArray_GETPTR1((PyArrayObject*)chroms, i);
-            cchroms[i] = PyString_AsString(PyArray_GETITEM((PyArrayObject*)chroms, foo));
+            cchroms[i] = getNumpyStr((PyArrayObject*)chroms, i);
 #endif
         }
         if(PyList_Check(starts)) {
@@ -968,6 +1207,9 @@ int PyAddIntervals(pyBigWigFile_t *self, PyObject *chroms, PyObject *starts, PyO
         self->lastTid = bwGetTid(bw, cchroms[n-1]);
         self->lastStart = uends[n-1];
     }
+    if(!PyList_Check(chroms)) {
+        for(i=0; i<n; i++) free(cchroms[i]);
+    }
     free(cchroms);
     free(ustarts);
     free(uends);
@@ -1389,6 +1631,11 @@ PyObject *pyBwAddEntries(pyBigWigFile_t *self, PyObject *args, PyObject *kwds) {
     PyObject *validate = Py_True;
     int desiredType;
 
+    if(!self->bw) {
+        PyErr_SetString(PyExc_RuntimeError, "The bigWig file handle is not open!");
+        return NULL;
+    }
+
     if(!PyArg_ParseTupleAndKeywords(args, kwds, "OO|OOOOO", kwd_list, &chroms, &starts, &ends, &values, &span, &step, &validate)) {
         PyErr_SetString(PyExc_RuntimeError, "Illegal arguments");
         return NULL;
@@ -1456,22 +1703,60 @@ static PyObject *pyBBGetEntries(pyBigWigFile_t *self, PyObject *args, PyObject *
     unsigned long startl, endl;
     char *chrom;
     static char *kwd_list[] = {"chrom", "start", "end", "withString", NULL};
-    PyObject *ret, *t;
+    PyObject *ret, *t, *starto = NULL, *endo = NULL;
     PyObject *withStringPy = Py_True;
     int withString = 1;
     bbOverlappingEntries_t *o;
 
+    if(!bw) {
+        PyErr_SetString(PyExc_RuntimeError, "The bigBed file handle is not open!");
+        return NULL;
+    }
+
     if(bw->type == 0) {
         PyErr_SetString(PyExc_RuntimeError, "bigWig files have no entries! Use 'intervals' or 'values' instead.");
         return NULL;  
     }
 
-    if(!PyArg_ParseTupleAndKeywords(args, kwds, "skk|O", kwd_list, &chrom, &startl, &endl, &withStringPy)) {
+    if(!PyArg_ParseTupleAndKeywords(args, kwds, "sOO|O", kwd_list, &chrom, &starto, &endo, &withStringPy)) {
         PyErr_SetString(PyExc_RuntimeError, "You must supply a chromosome, start and end position.\n");
         return NULL;
     }
 
     tid = bwGetTid(bw, chrom);
+
+#ifdef WITHNUMPY
+    if(PyArray_IsScalar(starto, Integer)) {
+        startl = (long) getNumpyL(starto);
+    } else
+#endif
+    if(PyLong_Check(starto)) {
+        startl = PyLong_AsLong(starto);
+#if PY_MAJOR_VERSION < 3
+    } else if(PyInt_Check(starto)) {
+        startl = PyInt_AsLong(starto);
+#endif
+    } else {
+        PyErr_SetString(PyExc_RuntimeError, "The start coordinate must be a number!");
+        return NULL;
+    }
+
+#ifdef WITHNUMPY
+    if(PyArray_IsScalar(endo, Integer)) {
+        endl = (long) getNumpyL(endo);
+    } else
+#endif
+    if(PyLong_Check(endo)) {
+        endl = PyLong_AsLong(endo);
+#if PY_MAJOR_VERSION < 3
+    } else if(PyInt_Check(endo)) {
+        endl = PyInt_AsLong(endo);
+#endif
+    } else {
+        PyErr_SetString(PyExc_RuntimeError, "The end coordinate must be a number!");
+        return NULL;
+    }
+
     if(endl == (unsigned long) -1 && tid != (uint32_t) -1) endl = bw->cl->len[tid];
     if(tid == (uint32_t) -1 || startl > end || endl > end) {
         PyErr_SetString(PyExc_RuntimeError, "Invalid interval bounds!");
@@ -1525,6 +1810,11 @@ static PyObject *pyBBGetSQL(pyBigWigFile_t *self, PyObject *args) {
     size_t len = 0;
     PyObject *o = NULL;
 
+    if(!bw) {
+        PyErr_SetString(PyExc_RuntimeError, "The bigBed file handle is not open!");
+        return NULL;
+    }
+
     if(!str) {
         Py_INCREF(Py_None);
         return Py_None;
@@ -1554,6 +1844,12 @@ static PyObject *pyIsBigWig(pyBigWigFile_t *self, PyObject *args) {
 
 static PyObject *pyIsBigBed(pyBigWigFile_t *self, PyObject *args) {
     bigWigFile_t *bw = self->bw;
+
+    if(!bw) {
+        PyErr_SetString(PyExc_RuntimeError, "The bigBed file handle is not open!");
+        return NULL;
+    }
+
     if(bw->type == 1) {
         Py_INCREF(Py_True);
         return Py_True;
@@ -1571,16 +1867,24 @@ static PyObject *pyIsBigBed(pyBigWigFile_t *self, PyObject *args) {
 
 #if PY_MAJOR_VERSION >= 3
 PyMODINIT_FUNC PyInit_pyBigWig(void) {
+#else
+PyMODINIT_FUNC initpyBigWig(void) {
+#endif
     PyObject *res;
     errno = 0; //just in case
 
+#if PY_MAJOR_VERSION >= 3
     if(Py_AtExit(bwCleanup)) return NULL;
-
     if(PyType_Ready(&bigWigFile) < 0) return NULL;
-    if(Py_AtExit(bwCleanup)) return NULL;
     if(bwInit(128000)) return NULL;
     res = PyModule_Create(&pyBigWigmodule);
     if(!res) return NULL;
+#else
+    if(Py_AtExit(bwCleanup)) return;
+    if(PyType_Ready(&bigWigFile) < 0) return;
+    if(bwInit(128000)) return;
+    res = Py_InitModule3("pyBigWig", bwMethods, "A module for handling bigWig files");
+#endif
 
     Py_INCREF(&bigWigFile);
     PyModule_AddObject(res, "pyBigWig", (PyObject *) &bigWigFile);
@@ -1592,25 +1896,14 @@ PyMODINIT_FUNC PyInit_pyBigWig(void) {
 #else
     PyModule_AddIntConstant(res, "numpy", 0);
 #endif
+#ifdef NOCURL
+    PyModule_AddIntConstant(res, "remote", 0);
+#else
+    PyModule_AddIntConstant(res, "remote", 1);
+#endif
+    PyModule_AddStringConstant(res, "__version__", pyBigWigVersion);
 
+#if PY_MAJOR_VERSION >= 3
     return res;
-}
-#else
-//Python2 initialization
-PyMODINIT_FUNC initpyBigWig(void) {
-    PyObject *res;
-    errno=0; //Sometimes libpython2.7.so is missing some links...
-    if(Py_AtExit(bwCleanup)) return;
-    if(PyType_Ready(&bigWigFile) < 0) return;
-    if(bwInit(128000)) return; //This is temporary
-    res = Py_InitModule3("pyBigWig", bwMethods, "A module for handling bigWig files");
-
-#ifdef WITHNUMPY
-    //Add the numpy constant
-    import_array(); //Needed for numpy stuff to work
-    PyModule_AddIntConstant(res, "numpy", 1);
-#else
-    PyModule_AddIntConstant(res, "numpy", 0);
 #endif
 }
-#endif
