@@ -2,6 +2,8 @@
 #include <structmember.h>
 #include "bigWig.h"
 
+#define pyBigWigVersion "0.3.12"
+
 typedef struct {
     PyObject_HEAD
     bigWigFile_t *bw;
@@ -13,6 +15,7 @@ typedef struct {
 } pyBigWigFile_t;
 
 static PyObject *pyBwOpen(PyObject *self, PyObject *pyFname);
+static PyObject *pyBwEnter(pyBigWigFile_t *self, PyObject *args);
 static PyObject *pyBwClose(pyBigWigFile_t *pybw, PyObject *args);
 static PyObject *pyBwGetChroms(pyBigWigFile_t *pybw, PyObject *args);
 static PyObject *pyIsBigWig(pyBigWigFile_t *pybw, PyObject *args);
@@ -51,6 +54,10 @@ Arguments:\n\
 \n\
 >>> import pyBigWig\n\
 >>> bw = pyBigWig.open(\"some_file.bw\")\n"},
+    {NULL, NULL, 0, NULL}
+};
+
+static PyMethodDef bwObjMethods[] = {
     {"header", (PyCFunction)pyBwGetHeader, METH_VARARGS,
 "Returns the header of a bigWig file. This contains information such as: \n\
   * The version number of the file ('version').\n\
@@ -375,6 +382,8 @@ Keyword arguments:\n\
 >>> bw.addEntries([\"1\", \"1\", \"1\"], [0, 100, 125], ends=[5, 120, 126], values=[0.0, 1.0, 200.0], validate=False)\n\
 >>> bw.close()\n\
 >>> os.remove(oname)"},
+    {"__enter__", (PyCFunction)pyBwEnter, METH_NOARGS, NULL},
+    {"__exit__", (PyCFunction)pyBwClose, METH_VARARGS, NULL},
     {NULL, NULL, 0, NULL}
 };
 
@@ -433,7 +442,7 @@ static PyTypeObject bigWigFile = {
     0,                         /*tp_weaklistoffset*/
     0,                         /*tp_iter*/
     0,                         /*tp_iternext*/
-    bwMethods,                 /*tp_methods*/
+    bwObjMethods,                 /*tp_methods*/
     0,                         /*tp_members*/
     0,                         /*tp_getset*/
     0,                         /*tp_base*/
